/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 *  strict-local
 * @format
 */
"use strict";

function _slicedToArray(arr, i) {
  return (
    _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest()
  );
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;
  try {
    for (
      var _i = arr[Symbol.iterator](), _s;
      !(_n = (_s = _i.next()).done);
      _n = true
    ) {
      _arr.push(_s.value);
      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }
  return _arr;
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}

const WebSocketHMRClient = require("./WebSocketHMRClient");

const injectUpdate = require("./injectUpdate");

class HMRClient extends WebSocketHMRClient {
  constructor(url) {
    super(url);

    _defineProperty(this, "shouldApplyUpdates", true);

    _defineProperty(this, "outdatedModules", new Set());

    this.on("update", update => {
      if (this.shouldApplyUpdates) {
        injectUpdate(update);
      } else {
        // Remember if there were edits while Fast Refresh is off.
        // We'll want to warn about those modules if you turn it on.
        update.added.forEach(_ref => {
          let _ref2 = _slicedToArray(_ref, 1),
            id = _ref2[0];

          return this.outdatedModules.add(id);
        });
        update.modified.forEach(_ref3 => {
          let _ref4 = _slicedToArray(_ref3, 1),
            id = _ref4[0];

          return this.outdatedModules.add(id);
        });
      }
    });
  }
}

module.exports = HMRClient;
